=begin rdoc
ruby

Ulrich Mutze, www.urichmutze.de 

2008-12-07

This is free software. It comes without any warranty.
Please contact the author to report problems or to
ask questions.
=end 

require 'tk'
require 'interval'


module AppMath

=begin rdoc
class for representing a 'window to the real world'
With the x-direction there is associated a real interval, and with
the y-directon there is also associated a real interval. With hese intervals
there are associated integer numbers width and height, which govern the 
'pixelation' of these intervals.
=end
class Graph

  attr :fac, true
  attr :bgr_color
  attr :n_div, true
  attr :grid_color, true
  attr :text_color, true
  
  def bgr_color=(color)  
    @@bgr = color
    clear(@@bgr)
  end
 
  def initialize(parent,nx,ny,iv_x,iv_y)
    @fac = R.c 1.04
    @bgr_color = 'lightgreen'
    @n_div = 5
    @grid_color = 'darkgray'
    @text_color = 'black'
    
    @parent = parent
    @w = nx
    @h = ny
    set_size_x!(iv_x)
    set_size_y!(iv_y)
      # sets @a, @b, @c, @d, @ivx, @ivy
    col = @bgr_color
    @canvas = TkCanvas.new(parent){
      width nx
      height ny
      background col
    }
    @canvas.pack
  end
  
  def ivx; @ivx; end
  
  def ivy; @ivy; end

  def clear(color=@bgr_color)
    magic = 2 # not expected to need this
    TkcRectangle.new(@canvas, 0, 0, @w + magic, @h + magic, 
      'width'=> 0, 'fill' => color)
  end
  
  def grid_x(anArray, color=@grid_color)
    anArray.each{ |x|
      pa = ipos(x,@ivy.low)
      pb = ipos(x,@ivy.upp)
      line = TkcLine.new(@canvas,pa[0],pa[1],pb[0],pb[1])
      line.fill(color)
    }
  end
  
  def grid_y(anArray, color=@grid_color)
    anArray.each{ |y|
      pa = ipos(@ivx.low,y)
      pb = ipos(@ivx.upp,y)
      line = TkcLine.new(@canvas,pa[0],pa[1],pb[0],pb[1])
      line.fill(color)
    }
  end

  def draw(arr_x, arr_y, color, auto_adjust = true)
  #  clear(@bgr_color)
    nx = arr_x.length
    ny = arr_y.length
    n = Basics.inf(nx,ny)
    if auto_adjust
      ivx_e = Iv.from_array(arr_x) * @fac # the values to be shown
        # should stay a bit apat from the rim of the diagram
      ivy_e = Iv.from_array(arr_y) * @fac
      divx = ivx_e.axis_division(@n_div)
      divy = ivy_e.axis_division(@n_div)
      d_x = divx[0]
      d_y = divy[0]
      v_x = divx[1]
      v_y = divy[1]
      ivxNew = Iv.new(v_x.first, v_x.last)  
      ivyNew = Iv.new(v_y.first, v_y.last)
      set_size_x!(ivxNew)
      set_size_y!(ivyNew)
      grid_x(v_x, grid_color)
      grid_y(v_y, grid_color)
    end
    
    for i in 1...n
      xa = arr_x[i-1]
      xb = arr_x[i]
      ya = arr_y[i-1]
      yb = arr_y[i]
      pa = ipos(xa,ya)
      pb = ipos(xb,yb)
      line = TkcLine.new(@canvas,pa[0],pa[1],pb[0],pb[1])
      line.fill(color)
    end
    if @bgr_color != @text_color # only then writing the text works, and
      # so we have a method to avoid writing text
# some magic numbers here    
      t_x = (@w * 0.1).to_i 
      t_y = (@h * 0.1).to_i
      t_w = (@w * 0.75).to_i
      t_h = (@h * 0.02).to_i
      t_h = Basics.sup(t_h,10)

      font_text = "LucidaConsole "+t_h.to_s

      xl = @ivx.low.to_s
      xu = @ivx.upp.to_s
      yl = @ivy.low.to_s
      yu = @ivy.upp.to_s
    
      x_text=xl + " < x < " + xu + " , dx_grid = " + d_x.to_s
      y_text=yl + " < y < " + yu + " , dy_grid = " + d_y.to_s
      full_text = x_text + "\n\n" + y_text
    
      txt=TkcText.new(@canvas,t_x,t_y){
        anchor "nw"
        width t_w 
        text full_text
        font font_text
      }
      txt.fill(@text_color)
    end
  end
  
  def draw_func(f, n, color = 'red', auto_adjust_y = true)
    xi = @ivx.low 
    arr_x = Array.new(n, xi)
    dx = @ivx.size/(n - 1.0)
    arr_y = Array.new(n, 0.0)
    for i in 0...n
      arr_x[i] = xi
      yi = f.at(xi)
      arr_y[i] = yi.to_f
      xi += dx
    end
    draw(arr_x, arr_y, color, auto_adjust_y)
  end
  
  def ipos x, y
    [@a + @b * x, @c + @d * y]
  end
  
  def set_size_x!(aIv)
    @ivx = aIv
    @b = (@w - 1.0)/@ivx.size
    @a = -@b * @ivx.low
  end
  
  def set_size_y!(aIv)
    @ivy = aIv
    @d = (1.0 - @h)/@ivy.size
    @c = -@d * @ivy.upp
  end
    
end 

# The code
#   root = TkRoot.new{ title 'Test of ...'}
#   g = Graphs.new(root,3,2,1000,700)
# generates a 2 times 3 matrix of Graph-objects
#   g.at(0,0), g.at(0,1), g.at(0,2)
#   g.at(1,0), g.at(1,1), g.at(1,2)
# which together make up a rectangle of 1000 pixels in x-direction
# and 700 pixels in y-direction. Notice that the matrix is a grid 
# with 3 items in x-direction and 2 items in y-direction.
# Since these items are of class Graph, the can be used for graphical
# representations of arrays and of functions without needing suport fom 
# the present class Graphs.
class Graphs

  attr_reader :px,:py,:mx,:my,:grs

  def initialize(parent,px,py,nx,ny)
    zero = R.c0
    one = R.c1
    @px = px
    @py = py
    @mx = nx / @px
    @my = ny / @py
    ivx = Iv.new(zero,one)
    ivy = Iv.new(zero,one)
    @grs = Array.new
    py.times{ |r|
      gr = Array.new
      px.times{ |c|
        fr = TkFrame.new(parent).grid('row' => r, 'column' => c)
        gr << Graph.new(fr,@mx,@my,ivx,ivy)
      }
      @grs << gr
    }
  end

# Access function. For a Graphs-object g, one has the Graph-objects
#   g.at(i,j), 0 <= i < p.py, 0 <= j < g.px . 
# All integer values are allowed and these then are understood 
# periodically.
  def at(i,j)
    return @grs[i.to_i%@py][j.to_i%@px]
  end

end

# function objects for testing Graph#draw_func
# with functions for which the accuracy of the functions
# is enforced

class Sin
  
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end

  def at(x)
    R.prec = @precLoc
    y = x.sin
    R.prec = @precMem; y
  end
end  

class Cos
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.cos
    R.prec = @precMem; y
  end
end  

class ArcSin
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.asin
    R.prec = @precMem; y
  end
end  

class Exp
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.exp
    R.prec = @precMem; y
  end
end  

class Log
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.log
    R.prec = @precMem; y
  end
end  

class ArcTan
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.atan
    R.prec = @precMem; y
  end
end 

class ArcCos
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.acos
    R.prec = @precMem; y
  end
end 

class ArcCot
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.acot
    R.prec = @precMem; y
  end
end 

class Erf
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.erf
    R.prec = @precMem; y
  end
end 

class Erfc
  def initialize(n = R.prec)
    @precMem = R.prec
    @precLoc = n
  end
  def at(x)
    R.prec = @precLoc
    y = x.erfc
    R.prec = @precMem; y
  end
end 


end # AppMath

