=begin rdoc
ruby

Ulrich Mutze www.ulrichmutze.de

2008-12-07

This is free software. It comes without any warranty.
Please contact the author to report problems or to
ask questions.
=end 

require 'appmath_basics.rb'

module AppMath

###################### classes R2 and Kep2D ##############################

# Real vector space of two dimensions
class R2

  attr :x, true
  attr :y, true
  
  def initialize(x,y)
    @x = x
    @y = y
  end
  
  def clone
    R.new(x,y)
  end
  
  def +(p)
    R2.new(x + p.x, y + p.y)
  end
  
  def -(p)
    R2.new(x - p.x, y - p.y)
  end
  
  def *(s)
    R2.new(x * s, y * s)
  end
  
  def -@
    R2.new(-x, -y)
  end
  
  def to_s
    res = "R2(" + x.to_s + ", " + y.to_s + ")"
  end
  
  def abs
    x.hypot(y)
  end

# abs squared
  def abs2
    x * x + y * y
  end

# scalar product
  def spr(p)
    x * p.x + y * p.y
  end

# unit vector
  def uv
    r = abs
    if r.zero?
      clone
    else 
      ri = r.inv
      self * ri
    end
  end
  
end

# Kepler problem in 2 dimensions
# mass of the test particle is 1.
# space-fixed central mass times constant of gravity is @g
class Kep2D

  def initialize(x,v,g)
    @t = R.c0
    @x = x
    @v = v
    @g = g
  end

# acceleration
  def acc
    r = @x.abs
    k = -@g * r**-3
    @x * k
  end 

# time step of the direct midpoint integrator, highly symmetric !
  def step!(dt)
    h = dt * 0.5
    @t += h
    @x += @v * h
    @v += acc * dt
    @x += @v * h
    @t += h
  end
  
  def to_s
    res = "x = " + @x.to_s + "\n" +
    "v = " + @v.to_s + "\n" +
    "t = " + @t.to_s
  end
  
  def get_x 
    @x.x
  end
  
  def get_y
    @x.y
  end
  
  def get_t
    @t
  end

# total energy
  def energy
    @v.abs2 * 0.5 - @g/@x.abs
  end

# angular momentum
  def ang_mom
    @x.x * @v.y - @x.y * @v.x
  end

# Runge-Lenz vector
  def lenz
    @x * @v.abs2 - @x * @v.spr(@x) - @x.uv * @g 
  end
  
end # class Kep2D 

end # AppMath

