=begin rdoc
ruby

Ulrich Mutze www.ulrichmutze.de

2008-11-30

Experimentation with Singular Value Decomposition in R

This is free software. It comes without any warranty.
Please contact the author to report problems or to
ask questions.
=end 

require 'linalg'
require 'cnum'

include AppMath

###################### input area ########################################

# selection parameter
sel = 3
prec = 100

verbose = false
complex = true

################### end of input area ####################################


R.prec = prec

case sel
when 1 
  n = 20
  zero = C.zero
  
  t1 = Time.now
  a = Mat.tob(n,12, complex)
  w = Vec.new(n,zero)
  v = Mat.new(n,n,zero)
  
  if verbose
    a.prn("a_input")
  end
  aMem = a.clone
  
  if complex
    csvdcmp(a,w,v)
  else
    Mat.svdcmp(a,w,v)
  end
  
  u = a
  
  aRec = u * Mat.new(w) * v.trp
  
  utu = u.trp * u
  vtv = v.trp * v
  
  utu1 = utu.to_1
  vtv1 = vtv.to_1
  
  if verbose
    utu.prn("utu")
    vtv.prn("vtv")
  end
  
  s = R.c0
  
  l = aMem
  r = aRec
  ds = l.dis(r)
  ds.prn("ds1")
  s += ds
  
  l = utu
  r = utu1
  ds = l.dis(r)
  ds.prn("ds2")
  s += ds
  
  l = vtv
  r = vtv1
  ds = l.dis(r)
  ds.prn("ds3")
  s += ds
  
  t2 = Time.now
  
  if verbose
    a.prn("a_output")
    puts
    w.prn("w_output")
    puts
    v.prn("v_output")
  end
  puts
  s.prn("error sum")
  (t2-t1).prn("computation time")
  
when 2
  Vec.test(10,verbose,complex)
  
when 3
  Mat.test(10,verbose,complex)  
  
when 4
  R.test(137,verbose)  
  
when 5
  C.test(137,verbose)  
  
when 6
  n = 16
  w = Vec.tob(n,137)
  w.prn("w")
  wpi = w.pseudo_inv
  wpi.prn("wpi")
  
when 7
  t1 = Time.now
  x = R.new("1.23456789E123")
  diff = (x.sin**2 + x.cos**2 - 1).abs.log10.round
  t2 = Time.now
  dt = t2-t1
  puts "diff = #{diff}, computation time was #{dt} s"
  
when 8
# input
  prec = 9
  dim = 32
  i = 137
  dig_round = 6
  complex = false
# computation  
  R.prec = prec
  x = Vec.tob(dim,i,complex)
  x.each{ |x|
    puts x.round(dig_round).to_s
  }
  
when 9
  prec = "f"
  R.prec = prec
  n = 20
  for i in 1..n
 #   puts R.ran(i).to_s
    puts R.tob(i).to_s
  end

when 10
  for i in 0..8
    R.prec = 20*i
    # some computation, e. g.
    x = [R.c2, R.i2, R.i(0.33333),R.ran(137), R.tob(3)] 
    puts "precision set to #{R.prec}"
    x.each{ |x|
      diff = x.sin**2 +x.cos**2 - 1
      puts "x = #{x.round(6)}, diff = #{diff}"
    }
  end
  
else 
  puts "Not even under construction"
  
end # case

