=begin rdoc
ruby

Ulrich Mutze, www.ulrichmutze.de

Started 2008-12-09

This is free software. It comes without any warranty.
Please contact the author to report problems or to
ask questions.
=end 

require 'interval'

module AppMath

=begin rdoc
Class of random generators.
The real numbers delivered by different random generators are intended
to be statistically independent. Each random generator has its own 
interval within which the output values are uniformly distributed.
=end

class Ran

  attr_reader :iv
  @@seed = 137
  @@swing = 133216711

# The arguments have to determine an interval in which the random values
# lie. This is named @iv. If there is no argument we have @iv = [0,1].
# If there is one argument, it has to be an instance of Iv and then 
# becomes @iv. If there are two arguments they need to be convertible to
# R and then become the boundaries of @iv.
  def initialize(*arg)
    n = arg.size
    case n
    when 0
      @iv = Iv.new(0,1)
    when 1
      a0 = arg[0]
      fail "argument must be an Iv" unless a0.class == Iv
      @iv = a0
    when 2
      a0 = arg[0]; a1 = arg[1]
      @iv = Iv.new(a0,a1)
    else
      fail "0,1,or 2 arguments needed, not #{n}"
    end
    x = R.ran(@@seed)
    i = (@@swing * x).to_i
    @local_seed = i
    @@seed += 1
  end

# Returns a point (value) from interval @iv by
# 'random selection'. The underlying mechanism is the 
# sine-floor generator defined in method R.ran.
  def ran
    p = R.ran(@local_seed)
    @local_seed += 1
    @iv.put(p)
  end

# Returns the local seed for control purposes. Should not be needed.
  def local_seed
    @local_seed
  end

end # class Ran

end # AppMath
